@@ -4,7 +4,9 @@
 package org.gcube.portlets.user.td.gwtservice.client.rpc;
 
 import java.util.ArrayList;
+import java.util.HashMap;
 
+import org.gcube.portlets.user.td.gwtservice.shared.chart.ChartTopRatingSession;
 import org.gcube.portlets.user.td.gwtservice.shared.codelisthelper.CodelistMappingSession;
 import org.gcube.portlets.user.td.gwtservice.shared.csv.AvailableCharsetList;
 import org.gcube.portlets.user.td.gwtservice.shared.csv.CSVExportSession;
@@ -14,11 +16,15 @@ import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTServiceExcept
 import org.gcube.portlets.user.td.gwtservice.shared.extract.ExtractCodelistSession;
 import org.gcube.portlets.user.td.gwtservice.shared.file.FileUploadMonitor;
 import org.gcube.portlets.user.td.gwtservice.shared.file.HeaderPresence;
+import org.gcube.portlets.user.td.gwtservice.shared.geometry.GeometryCreatePointSession;
+import org.gcube.portlets.user.td.gwtservice.shared.geospatial.GeospatialCreateCoordinatesSession;
+import org.gcube.portlets.user.td.gwtservice.shared.geospatial.GeospatialDownscaleCSquareSession;
 import org.gcube.portlets.user.td.gwtservice.shared.history.OpHistory;
 import org.gcube.portlets.user.td.gwtservice.shared.history.RollBackSession;
 import org.gcube.portlets.user.td.gwtservice.shared.json.JSONExportSession;
 import org.gcube.portlets.user.td.gwtservice.shared.licenses.LicenceData;
 import org.gcube.portlets.user.td.gwtservice.shared.map.MapCreationSession;
+import org.gcube.portlets.user.td.gwtservice.shared.monitor.BackgroundOperationMonitor;
 import org.gcube.portlets.user.td.gwtservice.shared.monitor.BackgroundOperationMonitorSession;
 import org.gcube.portlets.user.td.gwtservice.shared.monitor.OperationMonitor;
 import org.gcube.portlets.user.td.gwtservice.shared.monitor.OperationMonitorSession;
@@ -27,13 +33,13 @@ import org.gcube.portlets.user.td.gwtservice.shared.sdmx.SDMXImportSession;
 import org.gcube.portlets.user.td.gwtservice.shared.share.ShareTabResource;
 import org.gcube.portlets.user.td.gwtservice.shared.share.ShareTemplate;
 import org.gcube.portlets.user.td.gwtservice.shared.source.SDMXRegistrySource;
+import org.gcube.portlets.user.td.gwtservice.shared.statistical.StatisticalOperationSession;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskResubmitSession;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskResumeSession;
 import org.gcube.portlets.user.td.gwtservice.shared.task.ValidationsTasksMetadata;
 import org.gcube.portlets.user.td.gwtservice.shared.template.TemplateApplySession;
 import org.gcube.portlets.user.td.gwtservice.shared.template.TemplateData;
 import org.gcube.portlets.user.td.gwtservice.shared.template.TemplateDeleteSession;
-import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.RefColumn;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.TabResource;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.TableData;
@@ -42,6 +48,7 @@ import org.gcube.portlets.user.td.gwtservice.shared.tr.batch.OccurrencesForRepla
 import org.gcube.portlets.user.td.gwtservice.shared.tr.batch.ReplaceBatchColumnSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.clone.CloneTabularResourceSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.column.AddColumnSession;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.column.ChangeColumnsPositionSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.column.DeleteColumnSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.column.LabelColumnSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.column.ReplaceColumnSession;
@@ -52,8 +59,11 @@ import org.gcube.portlets.user.td.gwtservice.shared.tr.normalization.Normalizati
 import org.gcube.portlets.user.td.gwtservice.shared.tr.open.TDOpenSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.paging.CodelistPagingLoadConfig;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.paging.CodelistPagingLoadResult;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.replacebyexternal.ReplaceByExternalSession;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.RemoveResourceSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.ResourceTDDescriptor;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.ResourceTDType;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.SaveResourceSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.rows.DeleteRowsSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.rows.DuplicatesSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.rows.EditRowSession;
@@ -67,6 +77,10 @@ import org.gcube.portlets.user.td.gwtservice.shared.tr.union.UnionSession;
 import org.gcube.portlets.user.td.gwtservice.shared.uriresolver.UriResolverSession;
 import org.gcube.portlets.user.td.gwtservice.shared.user.UserInfo;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnData;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnDataType;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.PeriodDataType;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ValueDataFormat;
 
 import com.google.gwt.user.client.rpc.RemoteService;
 import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;
@@ -99,13 +113,20 @@ public interface TDGWTService extends RemoteService {
 	 */
 	public UserInfo hello() throws TDGWTServiceException;
 
+	/**
+	 * Retrieve pending Tasks and set them as background tasks
+	 * 
+	 * @throws TDGWTServiceException
+	 */
+	public Integer pendingTasksRetrieve() throws TDGWTServiceException;
+
 	/**
 	 * Restore UI session
 	 * 
 	 * @return
 	 * @throws TDGWTServiceException
 	 */
-	public TRId restoreUISession() throws TDGWTServiceException;
+	public TRId restoreUISession(TRId startTRId) throws TDGWTServiceException;
 
 	/**
 	 * Resolve Uri
@@ -117,6 +138,46 @@ public interface TDGWTService extends RemoteService {
 	public String getUriFromResolver(UriResolverSession uriResolverSession)
 			throws TDGWTServiceException;
 
+	// PeriodDataType
+	/**
+	 * Get list of PeriodDataType
+	 * 
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<PeriodDataType> getPeriodDataTypes()
+			throws TDGWTServiceException;
+
+	/**
+	 * Retrieves the hierarchical relationship for the specific period data type
+	 * 
+	 * @param periodDataType
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<PeriodDataType> getHierarchicalRelationshipForPeriodDataTypes(
+			PeriodDataType periodDataType) throws TDGWTServiceException;
+
+	// ValueDataFormats
+	/**
+	 * Retrieve ValueDataFormat for all ColumnDataType
+	 * 
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public HashMap<ColumnDataType, ArrayList<ValueDataFormat>> getValueDataFormatsMap()
+			throws TDGWTServiceException;
+
+	/**
+	 * Retrieve ValueDataFormat for specific ColumnDataType
+	 * 
+	 * @param columnDataType
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<ValueDataFormat> getValueDataFormatsOfColumnDataType(
+			ColumnDataType columnDataType) throws TDGWTServiceException;
+
 	// TabularResource
 	/**
 	 * Get current TRId
@@ -153,6 +214,14 @@ public interface TDGWTService extends RemoteService {
 	public void setTabResourceInformation(TabResource tabResource)
 			throws TDGWTServiceException;
 
+	/**
+	 * Set tabular resource to final
+	 * 
+	 * @param trId
+	 * @throws TDGWTServiceException
+	 */
+	public void setTabResourceToFinal(TRId trId) throws TDGWTServiceException;
+
 	/**
 	 * Get informations on tabular resource
 	 * 
@@ -277,11 +346,11 @@ public interface TDGWTService extends RemoteService {
 	/**
 	 * Set Current Tabular Resource that passed as parameter if not null
 	 * 
-	 * 
 	 * @param activeTRId
+	 * @return
 	 * @throws TDGWTServiceException
 	 */
-	public void setActiveTabularResource(TRId activeTRId)
+	public TRId setActiveTabularResource(TRId activeTRId)
 			throws TDGWTServiceException;
 
 	// Task
@@ -338,6 +407,18 @@ public interface TDGWTService extends RemoteService {
 	public ArrayList<ColumnData> getValidationColumns(TRId trId,
 			String columnName) throws TDGWTServiceException;
 
+	/**
+	 * Returns ArrayList<ColumnData> that contains all the validation columns of
+	 * columnLocalId
+	 * 
+	 * @param columnLocalId
+	 * @param trId
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<ColumnData> getValidationColumns(String columnLocalId,
+			TRId trId) throws TDGWTServiceException;
+
 	// RollBack
 	/**
 	 * Call rollback on tabular resource
@@ -438,6 +519,16 @@ public interface TDGWTService extends RemoteService {
 	public void tdOpen(TDOpenSession tdOpenSession)
 			throws TDGWTServiceException;
 
+	/**
+	 * Retrieve Time Table Id
+	 * 
+	 * @param periodDataType
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public Long getTimeTableId(PeriodDataType periodDataType)
+			throws TDGWTServiceException;
+
 	/**
 	 * Retrieves the list of columns in the current table
 	 * 
@@ -455,6 +546,41 @@ public interface TDGWTService extends RemoteService {
 	public ArrayList<ColumnData> getColumns(TRId trId)
 			throws TDGWTServiceException;
 
+	/**
+	 * Retrieves the list of columns in the table provided by trId view columns
+	 * included
+	 * 
+	 * @param trId
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<ColumnData> getColumnWithViewColumnIncluded(TRId trId)
+			throws TDGWTServiceException;
+	
+	/**
+	 * Retrieves the list of columns in the table provided by trId 
+	 * only view columns in relationship are included.
+	 * TimeDimensionColumn and DimensionColumn are not included. 
+	 * 
+	 * 
+	 * @param trId
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<ColumnData> getColumnWithOnlyViewColumnInRel(TRId trId)
+			throws TDGWTServiceException;
+	
+	/**
+	 * Retrieves the list of columns in the table provided by trId for
+	 * statistical
+	 * 
+	 * @param trId
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public ArrayList<ColumnData> getColumnsForStatistical(TRId trId)
+			throws TDGWTServiceException;
+
 	/**
 	 * Retrieves the list of columns in the table provided by trId only CODE and
 	 * CODENAME types
@@ -724,6 +850,18 @@ public interface TDGWTService extends RemoteService {
 	public String startLabelColumn(LabelColumnSession labelColumnSession)
 			throws TDGWTServiceException;
 
+	/**
+	 * Start change columns position
+	 * 
+	 * 
+	 * @param changeColumnsPositionSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public void startChangeColumnsPosition(
+			ChangeColumnsPositionSession changeColumnsPositionSession)
+			throws TDGWTServiceException;
+
 	/**
 	 * Start split column
 	 * 
@@ -789,6 +927,17 @@ public interface TDGWTService extends RemoteService {
 	public String startReplaceColumn(ReplaceColumnSession replaceColumnSession)
 			throws TDGWTServiceException;
 
+	/**
+	 * Start replace column by external tabular resource
+	 * 
+	 * @param replaceByExternalSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startReplaceByExternal(
+			ReplaceByExternalSession replaceByExternalSession)
+			throws TDGWTServiceException;
+
 	// Templates
 	/**
 	 * Retrieves templates of user
@@ -956,10 +1105,22 @@ public interface TDGWTService extends RemoteService {
 	 * @return
 	 * @throws TDGWTServiceException
 	 */
-	public ArrayList<OperationMonitor> getBackgroundOperationMonitor(
+	public ArrayList<BackgroundOperationMonitor> getBackgroundOperationMonitor(
 			BackgroundOperationMonitorSession backgroundOperationMonitorSession)
 			throws TDGWTServiceException;
 
+	/**
+	 * 
+	 * Retrieve Background Operation Monitor for specific task
+	 * 
+	 * @param operationMonitorSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public OperationMonitor getBackgroundOperationMonitorForSpecificTask(
+			OperationMonitorSession operationMonitorSession)
+			throws TDGWTServiceException;
+
 	// File Upload Monitor
 	/**
 	 * Get File Upload Monitor during the file upload operation in Import CSV
@@ -992,6 +1153,24 @@ public interface TDGWTService extends RemoteService {
 	public ArrayList<ResourceTDDescriptor> getResourcesTDByType(TRId trId,
 			ResourceTDType resourceTDType) throws TDGWTServiceException;
 
+	/**
+	 * Remove resource
+	 * 
+	 * @param removeResourceSession
+	 * @throws TDGWTServiceException
+	 */
+	public void removeResource(RemoveResourceSession removeResourceSession)
+			throws TDGWTServiceException;
+
+	/**
+	 * Save resource on Workspace
+	 * 
+	 * @param saveResourceSession
+	 * @throws TDGWTServiceException
+	 */
+	public void saveResource(SaveResourceSession saveResourceSession)
+			throws TDGWTServiceException;
+
 	// GIS MAP
 	/**
 	 * Start Map Creation and invokes the client library
@@ -1003,4 +1182,63 @@ public interface TDGWTService extends RemoteService {
 	public String startMapCreation(MapCreationSession mapCreationSession)
 			throws TDGWTServiceException;
 
+	// Statistical
+	/**
+	 * 
+	 * @param statisticalOperationSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startStatisticalOperation(
+			StatisticalOperationSession statisticalOperationSession)
+			throws TDGWTServiceException;
+
+	// Chart
+	/**
+	 * Start Chart Top Rating Creation and invokes the client library
+	 * 
+	 * @param chartTopRatingSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startChartTopRating(
+			ChartTopRatingSession chartTopRatingSession)
+			throws TDGWTServiceException;
+
+	// Geospatial
+	/**
+	 * Start Geospatial Coordinates Creation
+	 * 
+	 * @param geospatialCreateCoordinatesSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startGeospatialCreateCoordinates(
+			GeospatialCreateCoordinatesSession geospatialCreateCoordinatesSession)
+			throws TDGWTServiceException;
+
+	/**
+	 * Start Downscale C-Square
+	 * 
+	 * 
+	 * @param geospatialDownscaleCSquareSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startGeospatialDownscaleCSquare(
+			GeospatialDownscaleCSquareSession geospatialDownscaleCSquareSession)
+			throws TDGWTServiceException;
+
+	// Geometry
+	/**
+	 * Point Creation
+	 * 
+	 * @param geometryCreatePointSession
+	 * @return
+	 * @throws TDGWTServiceException
+	 */
+	public String startGeometryCreatePoint(
+			GeometryCreatePointSession geometryCreatePointSession)
+			throws TDGWTServiceException;
+
 }
